@@ -5,9 +5,9 @@ use rustc::lint::*;
 use rustc::hir::*;
 use rustc::ty::{self, TyCtxt};
 use semver::Version;
-use syntax::ast::{Attribute, Lit, LitKind, MetaItemKind, NestedMetaItem, NestedMetaItemKind};
+use syntax::ast::{Attribute, AttrStyle, Lit, LitKind, MetaItemKind, NestedMetaItem, NestedMetaItemKind};
 use syntax::codemap::Span;
-use utils::{in_macro, match_def_path, paths, span_lint, span_lint_and_then, snippet_opt};
+use utils::{in_macro, last_line_of_span, match_def_path, opt_def_id, paths, snippet_opt, span_lint, span_lint_and_then, without_block_comments};
 
 /// **What it does:** Checks for items annotated with `#[inline(always)]`,
 /// unless the annotated function is empty or simply panics.
@@ -29,9 +29,9 @@ use utils::{in_macro, match_def_path, paths, span_lint, span_lint_and_then, snip
 /// #[inline(always)]
 /// fn not_quite_hot_code(..) { ... }
 /// ```
-declare_lint! {
+declare_clippy_lint! {
     pub INLINE_ALWAYS,
-    Warn,
+    pedantic,
     "use of `#[inline(always)]`"
 }
 
@@ -53,9 +53,9 @@ declare_lint! {
 /// #[allow(unused_import)]
 /// use foo::bar;
 /// ```
-declare_lint! {
+declare_clippy_lint! {
     pub USELESS_ATTRIBUTE,
-    Warn,
+    correctness,
     "use of lint attributes on `extern crate` items"
 }
 
@@ -72,18 +72,53 @@ declare_lint! {
 /// #[deprecated(since = "forever")]
 /// fn something_else(..) { ... }
 /// ```
-declare_lint! {
+declare_clippy_lint! {
     pub DEPRECATED_SEMVER,
-    Warn,
+    correctness,
     "use of `#[deprecated(since = \"x\")]` where x is not semver"
 }
 
+/// **What it does:** Checks for empty lines after outer attributes
+///
+/// **Why is this bad?**
+/// Most likely the attribute was meant to be an inner attribute using a '!'.
+/// If it was meant to be an outer attribute, then the following item
+/// should not be separated by empty lines.
+///
+/// **Known problems:** Can cause false positives.
+///
+/// From the clippy side it's difficult to detect empty lines between an attributes and the
+/// following item because empty lines and comments are not part of the AST. The parsing
+/// currently works for basic cases but is not perfect.
+///
+/// **Example:**
+/// ```rust
+/// // Bad
+/// #[inline(always)]
+///
+/// fn not_quite_good_code(..) { ... }
+///
+/// // Good (as inner attribute)
+/// #![inline(always)]
+///
+/// fn this_is_fine(..) { ... }
+///
+/// // Good (as outer attribute)
+/// #[inline(always)]
+/// fn this_is_fine_too(..) { ... }
+/// ```
+declare_clippy_lint! {
+    pub EMPTY_LINE_AFTER_OUTER_ATTR,
+    nursery,
+    "empty line after outer attribute"
+}
+
 #[derive(Copy, Clone)]
 pub struct AttrPass;
 
 impl LintPass for AttrPass {
     fn get_lints(&self) -> LintArray {
-        lint_array!(INLINE_ALWAYS, DEPRECATED_SEMVER, USELESS_ATTRIBUTE)
+        lint_array!(INLINE_ALWAYS, DEPRECATED_SEMVER, USELESS_ATTRIBUTE, EMPTY_LINE_AFTER_OUTER_ATTR)
     }
 }
 
@@ -94,13 +129,14 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for AttrPass {
                 return;
             }
             for item in items {
-                if_let_chain! {[
-                    let NestedMetaItemKind::MetaItem(ref mi) = item.node,
-                    let MetaItemKind::NameValue(ref lit) = mi.node,
-                    mi.name() == "since",
-                ], {
-                    check_semver(cx, item.span, lit);
-                }}
+                if_chain! {
+                    if let NestedMetaItemKind::MetaItem(ref mi) = item.node;
+                    if let MetaItemKind::NameValue(ref lit) = mi.node;
+                    if mi.ident.name == "since";
+                    then {
+                        check_semver(cx, item.span, lit);
+                    }
+                }
             }
         }
     }
@@ -110,8 +146,7 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for AttrPass {
             check_attrs(cx, item.span, &item.name, &item.attrs)
         }
         match item.node {
-            ItemExternCrate(_) |
-            ItemUse(_, _) => {
+            ItemExternCrate(_) | ItemUse(_, _) => {
                 for attr in &item.attrs {
                     if let Some(ref lint_list) = attr.meta_item_list() {
                         if let Some(name) = attr.name() {
@@ -125,17 +160,19 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for AttrPass {
                                             }
                                         }
                                     }
-                                    if let Some(mut sugg) = snippet_opt(cx, attr.span) {
-                                        if sugg.len() > 1 {
+                                    let line_span = last_line_of_span(cx, attr.span);
+
+                                    if let Some(mut sugg) = snippet_opt(cx, line_span) {
+                                        if sugg.contains("#[") {
                                             span_lint_and_then(
                                                 cx,
                                                 USELESS_ATTRIBUTE,
-                                                attr.span,
+                                                line_span,
                                                 "useless lint attribute",
                                                 |db| {
-                                                    sugg.insert(1, '!');
+                                                    sugg = sugg.replacen("#[", "#![", 1);
                                                     db.span_suggestion(
-                                                        attr.span,
+                                                        line_span,
                                                         "if you just forgot a `!`, use",
                                                         sugg,
                                                     );
@@ -171,7 +208,7 @@ fn is_relevant_item(tcx: TyCtxt, item: &Item) -> bool {
     if let ItemFn(_, _, _, _, _, eid) = item.node {
         is_relevant_expr(tcx, tcx.body_tables(eid), &tcx.hir.body(eid).value)
     } else {
-        false
+        true
     }
 }
 
@@ -196,14 +233,13 @@ fn is_relevant_block(tcx: TyCtxt, tables: &ty::TypeckTables, block: &Block) -> b
     if let Some(stmt) = block.stmts.first() {
         match stmt.node {
             StmtDecl(_, _) => true,
-            StmtExpr(ref expr, _) |
-            StmtSemi(ref expr, _) => is_relevant_expr(tcx, tables, expr),
+            StmtExpr(ref expr, _) | StmtSemi(ref expr, _) => is_relevant_expr(tcx, tables, expr),
         }
     } else {
-        block.expr.as_ref().map_or(
-            false,
-            |e| is_relevant_expr(tcx, tables, e),
-        )
+        block
+            .expr
+            .as_ref()
+            .map_or(false, |e| is_relevant_expr(tcx, tables, e))
     }
 }
 
@@ -211,15 +247,15 @@ fn is_relevant_expr(tcx: TyCtxt, tables: &ty::TypeckTables, expr: &Expr) -> bool
     match expr.node {
         ExprBlock(ref block) => is_relevant_block(tcx, tables, block),
         ExprRet(Some(ref e)) => is_relevant_expr(tcx, tables, e),
-        ExprRet(None) |
-        ExprBreak(_, None) => false,
-        ExprCall(ref path_expr, _) => {
-            if let ExprPath(ref qpath) = path_expr.node {
-                let fun_id = tables.qpath_def(qpath, path_expr.hir_id).def_id();
+        ExprRet(None) | ExprBreak(_, None) => false,
+        ExprCall(ref path_expr, _) => if let ExprPath(ref qpath) = path_expr.node {
+            if let Some(fun_id) = opt_def_id(tables.qpath_def(qpath, path_expr.hir_id)) {
                 !match_def_path(tcx, fun_id, &paths::BEGIN_PANIC)
             } else {
                 true
             }
+        } else {
+            true
         },
         _ => true,
     }
@@ -231,6 +267,32 @@ fn check_attrs(cx: &LateContext, span: Span, name: &Name, attrs: &[Attribute]) {
     }
 
     for attr in attrs {
+        if attr.is_sugared_doc {
+            return;
+        }
+        if attr.style == AttrStyle::Outer {
+            if attr.tokens.is_empty() || !is_present_in_source(cx, attr.span) {
+                return;
+            }
+
+            let begin_of_attr_to_item = Span::new(attr.span.lo(), span.lo(), span.ctxt());
+            let end_of_attr_to_item = Span::new(attr.span.hi(), span.lo(), span.ctxt());
+
+            if let Some(snippet) = snippet_opt(cx, end_of_attr_to_item) {
+                let lines = snippet.split('\n').collect::<Vec<_>>();
+                let lines = without_block_comments(lines);
+
+                if lines.iter().filter(|l| l.trim().is_empty()).count() > 2 {
+                    span_lint(
+                        cx,
+                        EMPTY_LINE_AFTER_OUTER_ATTR,
+                        begin_of_attr_to_item,
+                        "Found an empty line after an outer attribute. Perhaps you forgot to add a '!' to make it an inner attribute?"
+                    );
+                }
+            }
+        }
+
         if let Some(ref values) = attr.meta_item_list() {
             if values.len() != 1 || attr.name().map_or(true, |n| n != "inline") {
                 continue;
@@ -266,8 +328,22 @@ fn check_semver(cx: &LateContext, span: Span, lit: &Lit) {
 
 fn is_word(nmi: &NestedMetaItem, expected: &str) -> bool {
     if let NestedMetaItemKind::MetaItem(ref mi) = nmi.node {
-        mi.is_word() && mi.name() == expected
+        mi.is_word() && mi.ident.name == expected
     } else {
         false
     }
 }
+
+// If the snippet is empty, it's an attribute that was inserted during macro
+// expansion and we want to ignore those, because they could come from external
+// sources that the user has no control over.
+// For some reason these attributes don't have any expansion info on them, so
+// we have to check it this way until there is a better way.
+fn is_present_in_source(cx: &LateContext, span: Span) -> bool {
+    if let Some(snippet) = snippet_opt(cx, span) {
+        if snippet.is_empty() {
+            return false;
+        }
+    }
+    true
+}
