@@ -1,20 +1,30 @@
-#![feature(plugin)]
-#![plugin(clippy)]
+
+#![allow(print_literal)]
 #![warn(useless_format)]
 
+struct Foo(pub String);
+
+macro_rules! foo {
+  ($($t:tt)*) => (Foo(format!($($t)*)))
+}
+
 fn main() {
     format!("foo");
 
     format!("{}", "foo");
-    format!("{:?}", "foo"); // we only want to warn about `{}`
-    format!("{:+}", "foo"); // we only want to warn about `{}`
+    format!("{:?}", "foo"); // don't warn about debug
+    format!("{:8}", "foo");
+    format!("{:+}", "foo"); // warn when the format makes no difference
+    format!("{:<}", "foo"); // warn when the format makes no difference
     format!("foo {}", "bar");
     format!("{} bar", "foo");
 
     let arg: String = "".to_owned();
     format!("{}", arg);
-    format!("{:?}", arg); // we only want to warn about `{}`
-    format!("{:+}", arg); // we only want to warn about `{}`
+    format!("{:?}", arg); // don't warn about debug
+    format!("{:8}", arg);
+    format!("{:+}", arg); // warn when the format makes no difference
+    format!("{:<}", arg); // warn when the format makes no difference
     format!("foo {}", arg);
     format!("{} bar", arg);
 
@@ -31,4 +41,7 @@ fn main() {
     println!("foo {}", "foo");
     println!("{}", 42);
     println!("foo {}", 42);
+
+    // A format! inside a macro should not trigger a warning
+    foo!("should not warn");
 }
