@@ -70,8 +70,10 @@ use expr::rewrite_call;
 use macros::convert_try_mac;
 use rewrite::{Rewrite, RewriteContext};
 use shape::Shape;
-use utils::{first_line_width, last_line_extendable, last_line_width, mk_sp,
-            trimmed_last_line_width, wrap_str};
+use utils::{
+    first_line_width, last_line_extendable, last_line_width, mk_sp, trimmed_last_line_width,
+    wrap_str,
+};
 
 use std::borrow::Cow;
 use std::cmp::min;
@@ -104,7 +106,7 @@ pub fn rewrite_chain(expr: &ast::Expr, context: &RewriteContext, shape: Shape) -
     };
     let parent_rewrite = parent
         .rewrite(context, parent_shape)
-        .map(|parent_rw| parent_rw + &repeat_try(prefix_try_num))?;
+        .map(|parent_rw| parent_rw + &"?".repeat(prefix_try_num))?;
     let parent_rewrite_contains_newline = parent_rewrite.contains('\n');
     let is_small_parent = parent_rewrite.len() <= context.config.tab_spaces();
 
@@ -263,7 +265,8 @@ pub fn rewrite_chain(expr: &ast::Expr, context: &RewriteContext, shape: Shape) -
         nested_shape.indent.to_string_with_newline(context.config)
     };
 
-    let first_connector = if is_small_parent || fits_single_line
+    let first_connector = if is_small_parent
+        || fits_single_line
         || last_line_extendable(&parent_rewrite)
         || context.config.indent_style() == IndentStyle::Visual
     {
@@ -273,7 +276,8 @@ pub fn rewrite_chain(expr: &ast::Expr, context: &RewriteContext, shape: Shape) -
     };
 
     let result = if is_small_parent && rewrites.len() > 1 {
-        let second_connector = if fits_single_line || rewrites[1] == "?"
+        let second_connector = if fits_single_line
+            || rewrites[1] == "?"
             || last_line_extendable(&rewrites[0])
             || context.config.indent_style() == IndentStyle::Visual
         {
@@ -297,7 +301,7 @@ pub fn rewrite_chain(expr: &ast::Expr, context: &RewriteContext, shape: Shape) -
             join_rewrites(&rewrites, &connector)
         )
     };
-    let result = format!("{}{}", result, repeat_try(suffix_try_num));
+    let result = format!("{}{}", result, "?".repeat(suffix_try_num));
     if context.config.indent_style() == IndentStyle::Visual {
         wrap_str(result, context.config.max_width(), shape)
     } else {
@@ -316,12 +320,6 @@ fn chain_only_try(exprs: &[ast::Expr]) -> bool {
     })
 }
 
-// Try to rewrite and replace the last non-try child. Return `true` if
-// replacing succeeds.
-fn repeat_try(try_count: usize) -> String {
-    iter::repeat("?").take(try_count).collect::<String>()
-}
-
 fn rewrite_try(
     expr: &ast::Expr,
     try_count: usize,
@@ -329,7 +327,7 @@ fn rewrite_try(
     shape: Shape,
 ) -> Option<String> {
     let sub_expr = expr.rewrite(context, shape.sub_width(try_count)?)?;
-    Some(format!("{}{}", sub_expr, repeat_try(try_count)))
+    Some(format!("{}{}", sub_expr, "?".repeat(try_count)))
 }
 
 fn join_rewrites(rewrites: &[String], connector: &str) -> String {
@@ -340,7 +338,7 @@ fn join_rewrites(rewrites: &[String], connector: &str) -> String {
         if rewrite != "?" {
             result.push_str(connector);
         }
-        result.push_str(&rewrite[..]);
+        result.push_str(&rewrite);
     }
 
     result
@@ -399,9 +397,9 @@ fn pop_expr_chain(expr: &ast::Expr, context: &RewriteContext) -> Option<ast::Exp
         ast::ExprKind::MethodCall(_, ref expressions) => {
             Some(convert_try(&expressions[0], context))
         }
-        ast::ExprKind::TupField(ref subexpr, _)
-        | ast::ExprKind::Field(ref subexpr, _)
-        | ast::ExprKind::Try(ref subexpr) => Some(convert_try(subexpr, context)),
+        ast::ExprKind::Field(ref subexpr, _) | ast::ExprKind::Try(ref subexpr) => {
+            Some(convert_try(subexpr, context))
+        }
         _ => None,
     }
 }
@@ -444,21 +442,30 @@ fn rewrite_chain_subexpr(
                 },
                 _ => &[],
             };
-            rewrite_method_call(segment.identifier, types, expressions, span, context, shape)
+            rewrite_method_call(segment.ident, types, expressions, span, context, shape)
         }
-        ast::ExprKind::Field(_, ref field) => rewrite_element(format!(".{}", field.node)),
-        ast::ExprKind::TupField(ref expr, ref field) => {
-            let space = match expr.node {
-                ast::ExprKind::TupField(..) => " ",
-                _ => "",
+        ast::ExprKind::Field(ref nested, ref field) => {
+            let space = if is_tup_field_access(expr) && is_tup_field_access(nested) {
+                " "
+            } else {
+                ""
             };
-            rewrite_element(format!("{}.{}", space, field.node))
+            rewrite_element(format!("{}.{}", space, field.name))
         }
         ast::ExprKind::Try(_) => rewrite_element(String::from("?")),
         _ => unreachable!(),
     }
 }
 
+fn is_tup_field_access(expr: &ast::Expr) -> bool {
+    match expr.node {
+        ast::ExprKind::Field(_, ref field) => {
+            field.name.to_string().chars().all(|c| c.is_digit(10))
+        }
+        _ => false,
+    }
+}
+
 // Determines if we can continue formatting a given expression on the same line.
 fn is_continuable(expr: &ast::Expr) -> bool {
     match expr.node {
