@@ -14,22 +14,21 @@ use std::fs::{self, File};
 use std::io::{self, BufWriter, Read, Write};
 use std::path::Path;
 
-use checkstyle::{output_checkstyle_file, output_footer, output_header};
+use checkstyle::output_checkstyle_file;
 use config::{Config, NewlineStyle, WriteMode};
-use rustfmt_diff::{make_diff, print_diff, Mismatch};
+use rustfmt_diff::{make_diff, output_modified, print_diff, Mismatch};
 use syntax::codemap::FileName;
 
-// A map of the files of a crate, with their new content
-pub type FileMap = Vec<FileRecord>;
-
-pub type FileRecord = (FileName, String);
+#[cfg(test)]
+use FileRecord;
 
 // Append a newline to the end of each file.
 pub fn append_newline(s: &mut String) {
     s.push_str("\n");
 }
 
-pub fn write_all_files<T>(
+#[cfg(test)]
+pub(crate) fn write_all_files<T>(
     file_map: &[FileRecord],
     out: &mut T,
     config: &Config,
@@ -37,11 +36,15 @@ pub fn write_all_files<T>(
 where
     T: Write,
 {
-    output_header(out, config.write_mode()).ok();
+    if config.write_mode() == WriteMode::Checkstyle {
+        ::checkstyle::output_header(out)?;
+    }
     for &(ref filename, ref text) in file_map {
         write_file(text, filename, out, config)?;
     }
-    output_footer(out, config.write_mode()).ok();
+    if config.write_mode() == WriteMode::Checkstyle {
+        ::checkstyle::output_footer(out)?;
+    }
 
     Ok(())
 }
@@ -120,7 +123,7 @@ where
     match config.write_mode() {
         WriteMode::Replace => {
             let filename = filename_to_path();
-            if let Ok((ori, fmt)) = source_and_formatted_text(text, &filename, config) {
+            if let Ok((ori, fmt)) = source_and_formatted_text(text, filename, config) {
                 if fmt != ori {
                     // Do a little dance to make writing safer - write to a temp file
                     // rename the original to a .bk, then rename the temp file to the
@@ -141,7 +144,7 @@ where
         WriteMode::Overwrite => {
             // Write text directly over original file if there is a diff.
             let filename = filename_to_path();
-            let (source, formatted) = source_and_formatted_text(text, &filename, config)?;
+            let (source, formatted) = source_and_formatted_text(text, filename, config)?;
             if source != formatted {
                 let file = File::create(filename)?;
                 write_system_newlines(file, text, config)?;
@@ -156,22 +159,45 @@ where
         }
         WriteMode::Diff => {
             let filename = filename_to_path();
-            if let Ok((ori, fmt)) = source_and_formatted_text(text, &filename, config) {
+            if let Ok((ori, fmt)) = source_and_formatted_text(text, filename, config) {
                 let mismatch = make_diff(&ori, &fmt, 3);
                 let has_diff = !mismatch.is_empty();
                 print_diff(
                     mismatch,
                     |line_num| format!("Diff in {} at line {}:", filename.display(), line_num),
-                    config.color(),
+                    config,
                 );
                 return Ok(has_diff);
             }
         }
+        WriteMode::Modified => {
+            let filename = filename_to_path();
+            if let Ok((ori, fmt)) = source_and_formatted_text(text, filename, config) {
+                let mismatch = make_diff(&ori, &fmt, 0);
+                let has_diff = !mismatch.is_empty();
+                output_modified(out, mismatch);
+                return Ok(has_diff);
+            }
+        }
         WriteMode::Checkstyle => {
             let filename = filename_to_path();
-            let diff = create_diff(&filename, text, config)?;
+            let diff = create_diff(filename, text, config)?;
             output_checkstyle_file(out, filename, diff)?;
         }
+        WriteMode::Check => {
+            let filename = filename_to_path();
+            if let Ok((ori, fmt)) = source_and_formatted_text(text, filename, config) {
+                let mismatch = make_diff(&ori, &fmt, 3);
+                let has_diff = !mismatch.is_empty();
+                print_diff(
+                    mismatch,
+                    |line_num| format!("Diff in {} at line {}:", filename.display(), line_num),
+                    config,
+                );
+                return Ok(has_diff);
+            }
+        }
+        WriteMode::None => {}
     }
 
     // when we are not in diff mode, don't indicate differing files
