@@ -17,9 +17,11 @@ use attr::*;
 use codemap::{LineRangeUtils, SpanUtils};
 use comment::{CodeCharKind, CommentCodeSlices, FindUncommented};
 use config::{BraceStyle, Config};
-use items::{format_impl, format_trait, format_trait_alias, is_mod_decl, is_use_item,
-            rewrite_associated_impl_type, rewrite_associated_type, rewrite_extern_crate,
-            rewrite_type_alias, FnSig, StaticParts, StructParts};
+use items::{
+    format_impl, format_trait, format_trait_alias, is_mod_decl, is_use_item,
+    rewrite_associated_impl_type, rewrite_associated_type, rewrite_extern_crate,
+    rewrite_type_alias, FnSig, StaticParts, StructParts,
+};
 use macros::{rewrite_macro, rewrite_macro_def, MacroPosition};
 use rewrite::{Rewrite, RewriteContext};
 use shape::{Indent, Shape};
@@ -301,6 +303,13 @@ impl<'b, 'a: 'b> FmtVisitor<'a> {
         let filtered_attrs;
         let mut attrs = &item.attrs;
         match item.node {
+            // For use items, skip rewriting attributes. Just check for a skip attribute.
+            ast::ItemKind::Use(..) => {
+                if contains_skip(attrs) {
+                    self.push_skipped_with_span(item.span());
+                    return;
+                }
+            }
             // Module is inline, in this case we treat it like any other item.
             _ if !is_mod_decl(item) => {
                 if self.visit_attrs(&item.attrs, ast::AttrStyle::Outer) {
